@@ -118,7 +118,8 @@ if os.path.exists(ssl_paths.openssl_capath):
         (ssl_paths.openssl_capath, os.path.join('etc', 'ssl', 'certs')))
 
 version = Popen(
-    'python setup.py --version', stdout=PIPE, shell=True).stdout.read()
+    'python setup.py --version', stdout=PIPE, shell=True, encoding='utf-8'
+    ).stdout.read()
 version = version.strip()
 
 setup(name='tryton',
